@@ -221,7 +221,6 @@ xfce_settings_manager_dialog_init (XfceSettingsManagerDialog *dialog)
     GtkWidget *viewport;
     GtkWidget *image;
     GtkWidget *button;
-    GList     *children;
     gchar     *path;
 
     dialog->channel = xfconf_channel_get ("xfce4-settings-manager");
@@ -265,7 +264,7 @@ xfce_settings_manager_dialog_init (XfceSettingsManagerDialog *dialog)
     dialog->filter_entry = entry = gtk_entry_new ();
     gtk_header_bar_pack_end (GTK_HEADER_BAR (gtk_dialog_get_header_bar (GTK_DIALOG (dialog))), entry);
     gtk_widget_set_valign (entry, GTK_ALIGN_CENTER);
-    gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, "edit-find");
+    gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, "edit-find-symbolic");
     gtk_entry_set_icon_activatable (GTK_ENTRY (entry), GTK_ENTRY_ICON_SECONDARY, FALSE);
     g_signal_connect (G_OBJECT (entry), "changed",
         G_CALLBACK (xfce_settings_manager_dialog_entry_changed), dialog);
@@ -301,7 +300,7 @@ xfce_settings_manager_dialog_init (XfceSettingsManagerDialog *dialog)
     /* pluggable dialog scrolled window and viewport */
     dialog->socket_scroll = scroll = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_NONE);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_box_pack_start (GTK_BOX (dialog_vbox), scroll, TRUE, TRUE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (scroll), 0);
 
@@ -445,7 +444,7 @@ xfce_settings_manager_dialog_set_title (XfceSettingsManagerDialog *dialog,
     g_return_if_fail (XFCE_IS_SETTINGS_MANAGER_DIALOG (dialog));
 
     if (icon_name == NULL)
-        icon_name = "preferences-desktop";
+        icon_name = "org.xfce.settings.manager";
     if (title == NULL)
         title = _("Settings");
     if (subtitle == NULL)
@@ -727,7 +726,7 @@ xfce_settings_manager_dialog_entry_changed (GtkWidget                 *entry,
         {
             gtk_entry_set_icon_from_icon_name (GTK_ENTRY (dialog->filter_entry),
                 GTK_ENTRY_ICON_SECONDARY,
-                filter_text == NULL ? "edit-find" : "edit-clear");
+                filter_text == NULL ? "edit-find-symbolic" : "edit-clear-symbolic");
             gtk_entry_set_icon_activatable (GTK_ENTRY (dialog->filter_entry),
                 GTK_ENTRY_ICON_SECONDARY, filter_text != NULL);
         }
@@ -1249,10 +1248,12 @@ xfce_settings_manager_dialog_menu_reload (XfceSettingsManagerDialog *dialog)
     GList               *lnext;
     GarconMenuDirectory *directory;
     GList               *items, *lp;
+    GList               *keywords, *kli;
     gint                 i = 0;
     gchar               *item_text;
     gchar               *normalized;
     gchar               *filter_text;
+    GString             *item_keywords;
     DialogCategory      *category;
 
     g_return_if_fail (XFCE_IS_SETTINGS_MANAGER_DIALOG (dialog));
@@ -1298,10 +1299,19 @@ xfce_settings_manager_dialog_menu_reload (XfceSettingsManagerDialog *dialog)
                 items = g_list_sort (items, xfce_settings_manager_dialog_menu_sort);
                 for (lp = items; lp != NULL; lp = lp->next)
                 {
-                    /* create independent search string */
-                    item_text = g_strdup_printf ("%s\n%s",
+                    /* create independent search string based on name, comment and keywords */
+                    keywords = garcon_menu_item_get_keywords (lp->data);
+                    item_keywords = g_string_new (NULL);
+                    for (kli = keywords; kli != NULL; kli = kli->next)
+                    {
+                        g_string_append (item_keywords, kli->data);
+                    }
+                    item_text = g_strdup_printf ("%s\n%s\n%s",
                         garcon_menu_item_get_name (lp->data),
-                        garcon_menu_item_get_comment (lp->data));
+                        garcon_menu_item_get_comment (lp->data),
+                        item_keywords->str);
+                    g_string_free (item_keywords, TRUE);
+                    g_list_free (kli);
                     normalized = g_utf8_normalize (item_text, -1, G_NORMALIZE_DEFAULT);
                     g_free (item_text);
                     filter_text = g_utf8_casefold (normalized, -1);
